unit ifpsstdctrlx;
{
    Standard Controls library for Innerfuse Pascal Script
    Copyright(c) 2001 by Alain Falanga (afalanga@ifrance.com)

    Portions by Carlo Kok (ckok.1@hccnet.nl)
}
interface
uses
  ifspas, ifs_var, ifs_utl, ifsdfrm;

procedure RegisterStdCtrlXLibrary(Sender: TIfPasScript);
{
Registers:
  TProgressBarClass = Class(TControl)
    constructor Create(Aparent: TControl);
    property Position: integer read GetPosition write SetPosition;
    property Max: integer read GetMax write SetMax;
    property Min: integer read GetMin write SetMin;
  end;
  TImageClass = class(TControl)
    constructor Create(Aparent: TControl);
    property ImageFile: String read GetImage write SetImage;
  end;
  TOnTimerEvent = procedure (Sender: TObject);
  TTimerClass = class(TObject)
    constructor Create;
    property Interval: integer read GetInterval write SetInterval;
    property Enabled: boolean read GetEnabled write SetEnabled;
    property OnTimer: TOnTimerEvent read FOnTimer write FOnTimer;
  end;


}
implementation
uses
  Controls, Classes, ComCtrls, ExtCtrls;
const
// Declaration
  TProgressBarClass = 'class(TControl)'+
    'private '+
    'function GetPosition: integer;'+
    'procedure SetPosition(i: integer);'+
    'private function GetMax: integer;'+
    'procedure SetMax(i: integer);'+
    'private function GetMin: integer;'+
    'procedure SetMin(i: integer);'+
    'public constructor Create(Aparent: TControl);'+
    'property Position: integer read GetPosition write SetPosition;'+
    'property Max: integer read GetMax write SetMax;'+
    'property Min: integer read GetMin write SetMin;'+
    'end;';
  TImageClass = 'class(TControl)'+
    'private '+
    'function GetImage: String;'+
    'procedure SetImage(b: string);'+
    'public constructor Create(Aparent: TControl);'+
    'property ImageFile: String read GetImage write SetImage;'+
    'end;';
  TOnTimerEventType = 'procedure (Sender: TObject);';
  TTimerClass = 'class(TObject)'+
    'private '+
    'FOntimer: TOnTimerEvent;'+
    'FControl: ResourcePointer;'+
    'function GetInterval: integer;'+
    'procedure SetInterval(i: integer);'+
    'private function GetEnabled: boolean;'+
    'procedure SetEnabled(b: boolean);'+
    'public constructor Create;'+
    'property Interval: integer read GetInterval write SetInterval;'+
    'property Enabled: boolean read GetEnabled write SetEnabled;'+
    'property OnTimer: TOnTimerEvent read FOnTimer write FOnTimer;'+
    'end;';

// Function feedback implementation

function TProgressBarProc(Sender: TIfPasScript; ScriptID: Pointer; proc: PProcedure; Params: PVariableManager; res: PIfVariant): TIfPasScriptError;
var
  Ctrl, Self: PIfVariant;
  v: PVariableManager;
begin
  Self := Vm_Get(Params, 0);
  if (not GetClassVariable(Self, proc^.ClassType^.Ext, 'FCONTROL', Ctrl, True)) then begin
    TProgressBarProc := ENotSupported; // internal error
    exit;
  end;
  TProgressBarProc := ENoError;
  if proc^.Name = '!GETMAX' then begin
    if Ctrl^.Cv_Int1 = nil then begin
      TProgressBarProc := ENotSupported;
      exit;
    end;
    SetInteger(res, TProgressBar(Ctrl^.Cv_Int1).Max);
  end else if proc^.Name = '!SETMAX' then begin
    if Ctrl^.Cv_Int1 = nil then begin
      TProgressBarProc := ENotSupported;
      exit;
    end;
    TProgressBar(Ctrl^.Cv_Int1).Max := GetInteger(Vm_Get(Params, 1));
  end else if proc^.Name = '!GETMIN' then begin
    if Ctrl^.Cv_Int1 = nil then begin
      TProgressBarProc := ENotSupported;
      exit;
    end;
    SetInteger(res, TProgressBar(Ctrl^.Cv_Int1).Min);
  end else if proc^.Name = '!SETMIN' then begin
    if Ctrl^.Cv_Int1 = nil then begin
      TProgressBarProc := ENotSupported;
      exit;
    end;
    TProgressBar(Ctrl^.Cv_Int1).Min := GetInteger(Vm_Get(Params, 1));
  end else if proc^.Name = '!GETPOSITION' then begin
    if Ctrl^.Cv_Int1 = nil then begin
      TProgressBarProc := ENotSupported;
      exit;
    end;
    SetInteger(res, TProgressBar(Ctrl^.Cv_Int1).Position);
  end else if proc^.Name = '!SETPOSITION' then begin
    if Ctrl^.Cv_Int1 = nil then begin
      TProgressBarProc := ENotSupported;
      exit;
    end;
    TProgressBar(Ctrl^.Cv_Int1).Position := GetInteger(Vm_Get(Params, 1));
  end else if proc^.Name = '!CREATE' then begin
    Ctrl^.Cv_Int1 := TProgressBar.Create(nil);
    Ctrl^.Cv_Int2 := GetCV_Int2;
    TControl(Ctrl^.Cv_Int1).Tag := Integer(Self^.CV_Class);
    v := VM_Create(nil);
    Vm_Add(v, Self, 'SELF');
    Vm_Add(v, Sender.CopyVariant(Vm_Get(Params, 1)), '');
    Sender.RunScriptProc(GetInheritedProc(proc), v);
    VM_Delete(v, 0);
    VM_Destroy(v);
  end;
end;

function TImageProc(Sender: TIfPasScript; ScriptID: Pointer; proc: PProcedure; Params: PVariableManager; res: PIfVariant): TIfPasScriptError;
var
  Ctrl, Self: PIfVariant;
  v: PVariableManager;
begin
  Self := Vm_Get(Params, 0);
  if (not GetClassVariable(Self, proc^.ClassType^.Ext, 'FCONTROL', Ctrl, True)) then begin
    TImageProc := ENotSupported; // internal error
    exit;
  end;
  TImageProc := ENoError;
  if proc^.Name = '!GETIMAGE' then begin
    if Ctrl^.Cv_Int1 = nil then begin
      TImageProc := ENotSupported;
      exit;
    end;
    SetString(res, '');
  end else if proc^.Name = '!SETIMAGE' then begin
    if Ctrl^.Cv_Int1 = nil then begin
      TImageProc := ENotSupported;
      exit;
    end;
    try
      TImage(Ctrl^.Cv_Int1).Picture.LoadFromFile(GetString(Vm_Get(Params, 1)));
    except // needed because TIfPasScript does not handle exceptions so it will not free memory
    end;
  end else if proc^.Name = '!CREATE' then begin
    Ctrl^.Cv_Int1 := TImage.Create(nil);
    Ctrl^.Cv_Int2 := GetCV_Int2;
    TControl(Ctrl^.Cv_Int1).Tag := Integer(Self^.CV_Class);
    TControl(Ctrl^.Cv_Int1).Enabled := FALSE;
    v := VM_Create(nil);
    Vm_Add(v, Self, 'SELF');
    Vm_Add(v, Sender.CopyVariant(Vm_Get(Params, 1)), '');
    Sender.RunScriptProc(GetInheritedProc(proc), v);
    VM_Delete(v, 0);
    VM_Destroy(v);
  end;
end;

procedure MyOnTimer(ScriptEngine: TIFPasScript; Timer: TTimer);
var
  Vars: PVariableManager;
  E: PIfVariant;
begin
  if not GetClassVariable2(PCreatedClass(Timer.Tag), PCreatedClass(Timer.Tag)^.ClassType^.Ext, 'FONTIMER', E, True) then exit;
  if not assigned(E^.Cv_Proc) then exit;
  Vars := VM_Create(nil);
  Vm_Add(Vars, CreateCajVariant(PCreatedClass(Timer.Tag)^.ClassType), 'SELF')^.CV_Class := PCreatedClass(Timer.Tag);
  ScriptEngine.RunScriptProc(E^.Cv_Proc, Vars);
  VM_Destroy(Vars);
end;

function TTimerProc(Sender: TIfPasScript; ScriptID: Pointer; proc: PProcedure; Params: PVariableManager; res: PIfVariant): TIfPasScriptError;
var
  Ctrl, Self: PIfVariant;
begin
  Self := Vm_Get(Params, 0);
  if (not GetClassVariable(Self, proc^.ClassType^.Ext, 'FCONTROL', Ctrl, True)) then begin
    TTimerProc := ENotSupported; // internal error
    exit;
  end;
  TTimerProc := ENoError;
  if proc^.Name = '!GETINTERVAL' then begin
    if Ctrl^.Cv_Int1 = nil then begin
      TTimerProc := ENotSupported;
      exit;
    end;
    SetInteger(res, TTimer(Ctrl^.Cv_Int1).Interval);
  end else if proc^.Name = '!SETINTERVAL' then begin
    if Ctrl^.Cv_Int1 = nil then begin
      TTimerProc := ENotSupported;
      exit;
    end;
    TTimer(Ctrl^.Cv_Int1).Interval := GetInteger(Vm_Get(Params, 1));
  end else if proc^.Name = '!GETENABLED' then begin
    if Ctrl^.Cv_Int1 = nil then begin
      TTimerProc := ENotSupported;
      exit;
    end;
    SetBoolean(res, TTimer(Ctrl^.Cv_Int1).Enabled);
  end else if proc^.Name = '!SETENABLED' then begin
    if Ctrl^.Cv_Int1 = nil then begin
      TTimerProc := ENotSupported;
      exit;
    end;
    TTimer(Ctrl^.Cv_Int1).Enabled := GetBoolean(Vm_Get(Params, 1));
  end else if proc^.Name = '!CREATE' then begin
    Ctrl^.Cv_Int1 := AddTObjectFreeProc(Sender, TTimer.Create(nil));
    Ctrl^.Cv_Int2 := GetCV_Int2;
    TTimer(Ctrl^.Cv_Int1).Tag := Integer(Self^.CV_Class);
    TTimer(Ctrl^.Cv_Int1).Enabled := FALSE;
    TTimer(Ctrl^.Cv_Int1).OnTimer := TNotifyEvent(Proc2Method(@MyOnTimer, Sender));
  end else if Proc^.Name = '!DESTROY' then begin
    if Ctrl^.Cv_Int1 = nil then begin
      TTimerProc := ENotSupported;
      exit;
    end;
    Sender.RemoveResource(Ctrl^.CV_Int1);
    TTimer(Ctrl^.CV_Int1).Free;
  end;
end;

// Registering
procedure RegisterStdCtrlXLibrary(Sender: TIfPasScript);
begin
  Sender.AddClass('TPROGRESSBAR', TProgressBarClass, @TProgressBarProc);
  Sender.AddClass('TIMAGE', TImageClass, @TImageProc);
  Sender.AddType('TONTIMEREVENT', TOnTimerEventType);
  Sender.AddClass('TTIMER', TTimerClass, @TTimerProc);
end;

end.
