unit ifsdb;
{
  Database Library for Innerfuse Pascal Script 
  Made by Xaver Bauer <xb@condat.com>
}

interface
uses ifspas, ifs_var, ifs_utl, DB;

(*
CONST

  // -- Loacte Options -----
  loCaseInsensitive = 1;
  loPartialKey      = 2;

  // -- DataSet States -----
  dsInactive        = 0;
  dsBrowse          = 1;
  dsEdit            = 2;
  dsInsert          = 3;
  dsSetKey          = 4;
  dsCalcFields      = 5
  dsFilter          = 6;

TYPE
  TDataCon = Class(TObject)
  private
    function GetActive: Boolean;
    function GetBOF: Boolean;
    function GetCanModify: Boolean;
    function GetEOF: Boolean;
    function GetFieldCount: Integer;
    function GetFiltered: Boolean;
    function GetFilterOptions: Integer;
    function GetFilterText: string;
    function GetFound: Boolean;
    function GetModified: Boolean;
    function GetRecordCount: Integer;
    function GetRecordSize: Word;
    function GetState: Integer;

    function GetActive: Boolean;
    function GetFiltered: Boolean;
    function GetFilterOptions: Integer;
    function GetFilterText: string;
    procedure SetActive(const Value: Boolean);
    procedure SetFiltered(const Value: Boolean);
    procedure SetFilterOptions(const Value: Integer);
    procedure SetFilterText(const Value: string);
  protected
    FDataCon : ResourcePointer;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    // ----- Edit and Update Section ---------------
    procedure Append;
    procedure Insert;
    procedure Edit;
    procedure Delete;
    procedure Post; virtual;
    procedure Cancel; virtual;
    procedure ClearFields;

    procedure AppendRecord( Values: array of const );
    procedure InsertRecord( Values: array of const );
    procedure SetFields( Values: array of const );

    // ----- Open Close Section --------------------
    procedure Open;
    procedure Close;
    procedure Refresh;

    // ----- Locate Section ------------------------

    procedure First;
    procedure Last;
    procedure Next;
    procedure Prior;
    function MoveBy(Distance: Integer): Integer;

    function Locate( KeyFields: string; KeyValues : array of const;
                     LocateOptions: Integer ): Boolean;
    function Lookup( KeyFields: string; KeyValues : array of const;
                     ResultFields: string ): Variant;

    function FindFirst: Boolean;
    function FindLast: Boolean;
    function FindNext: Boolean;
    function FindPrior: Boolean;

    function GetBookmark: Pointer;
    procedure GotoBookmark( Bookmark: Pointer );
    procedure FreeBookmark( Bookmark: Pointer );
    function BookmarkValid( Bookmark: Pointer ): Boolean;

    // ----- Field-Data Section ------------------------

    function  GetField( FieldName: string ): String;
    procedure SetField( FieldName: string; S : String);

    // ----- Status and Information Section ------------------------

    function IsEmpty: Boolean;
    function IsSequenced: Boolean; virtual;

    // ----- Information Props (Read Only) -------------------------
    property Bof: Boolean read GetBOF;
    property Eof: Boolean read GetEOF;
    property CanModify: Boolean read GetCanModify;
    property FieldCount: Integer read GetFieldCount;
    property Found: Boolean read GetFound;
    property Modified: Boolean read GetModified;
    property RecordCount: Integer read GetRecordCount;
    property RecordSize: Word read GetRecordSize;
    property State: Integer read GetState;

    // ----- Filter Props (Read/Write) -------------------------
    property Filter: string read GetFilterText write SetFilterText;
    property Filtered: Boolean read GetFiltered write SetFiltered;
    property FilterOptions: Integer read GetFilterOptions write SetFilterOptions;

    property Active: Boolean read GetActive write SetActive;
  end;

*)
procedure RegisterDBLibrary(p: TIfPasScript);

implementation

CONST
  TDataConClass =
    'class(TObject)'+
    'private '+
    // ----- Propertys ReadOnly ---------------
    'function GetBOF: Boolean;'+
    'function GetCanModify: Boolean;'+
    'function GetEOF: Boolean;'+
    'function GetFieldCount: Integer;'+
    'function GetFound: Boolean;'+
    'function GetModified: Boolean;'+
    'function GetRecordCount: Integer;'+
    'function GetRecordSize: Word;'+
    'function GetState: Integer;'+
    // ----- Propertys Read/Write ---------------
    'function GetActive: Boolean;'+
    'function GetFiltered: Boolean;'+
    'function GetFilterOptions: Integer;'+
    'function GetFilterText: string;'+
    'procedure SetActive( B: Boolean);'+
    'procedure SetFiltered( B: Boolean);'+
    'procedure SetFilterOptions( I: Integer);'+
    'procedure SetFilterText( S: string);'+
    'protected '+
    'FDataCon : ResourcePointer;'+
    'public '+
    'constructor Create;'+
    'destructor Destroy; override;'+
    // ----- Edit and Update Section ---------------
    'procedure Append;'+
    'procedure Insert;'+
    'procedure Edit;'+
    'procedure Delete;'+
    'procedure Post;'+
    'procedure Cancel;'+
    'procedure ClearFields;'+
    'procedure AppendRecord( Values: tKeyValues );'+
    'procedure InsertRecord( Values: tKeyValues );'+
    'procedure SetFields( Values: tKeyValues );'+
    // ----- Open Close Section --------------------
    'procedure Open;'+
    'procedure Close;'+
    'procedure Refresh;'+
    // ----- Locate Section ------------------------
    'procedure First;'+
    'procedure Last;'+
    'procedure Next;'+
    'procedure Prior;'+
    'function MoveBy(Distance: Integer): Integer;'+
    'function Locate( KeyFields: string; KeyValues: tKeyValues; LocateOptions: Integer ): Boolean;'+
    'function Lookup( KeyFields: string; KeyValues: tKeyValues; ResultFields: string ): Variant;'+
    'procedure FindFirst;'+
    'procedure FindLast;'+
    'procedure FindNext;'+
    'procedure FindPrior;'+
    'function GetBookmark: TBookmark;'+
    'procedure GotoBookmark( Bookmark: TBookmark );'+
    'procedure FreeBookmark( Bookmark: TBookmark );'+
    'function BookmarkValid( Bookmark: TBookmark ): Boolean;'+
    // ----- Field-Data Section ------------------------
    'function  GetField( FieldName: string ): String;'+
    'procedure SetField( FieldName: string; S : String);'+
    'function  GetFieldAs( FieldName: string; FieldTyp: Integer): Variant;'+
    'procedure SetFieldAs( FieldName: string; V: Variant; FieldTyp: Integer);'+
    // ----- Status and Information Section ------------------------
    'function IsEmpty: Boolean;'+
    'function IsSequenced: Boolean; virtual;'+
    // ----- Information Props (Read Only) -------------------------
    'property Bof: Boolean read GetBOF;'+
    'property Eof: Boolean read GetEOF;'+
    'property CanModify: Boolean read GetCanModify;'+
    'property FieldCount: Integer read GetFieldCount;'+
    'property Found: Boolean read GetFound;'+
    'property Modified: Boolean read GetModified;'+
    'property RecordCount: Integer read GetRecordCount;'+
    'property RecordSize: Word read GetRecordSize;'+
    'property State: Integer read GetState;'+
    // ----- Filter Props (Read/Write) -------------------------
    'property Filter: string read GetFilterText write SetFilterText;'+
    'property Filtered: Boolean read GetFiltered write SetFiltered;'+
    'property FilterOptions: Integer read GetFilterOptions write SetFilterOptions;'+
    // ----- Open / Close Prop ---------------------------------
    'property Active: Boolean read GetActive write SetActive;'+
    'end;';

procedure TDataConFree(id: Pointer; Ctrl: Pointer);
begin
  if TObject(Ctrl) is TDataSet then begin
    with TObject(Ctrl) as TDataSet do begin
      if State in [dsEdit, dsInsert] then Cancel;
      Close;
    end;
    TDataSet(Ctrl).Free;
  end;
end;

function CheckActiveDataset(Script: TIfPasScript; ds : tDataSet): Boolean;
begin
  CheckActiveDataset:=True;
  if not ds.Active then Begin
    Script.RunError2(Script, ECustomError, 'Datenbank noch nicht geffnet');
    CheckActiveDataset := False;
  end;
end;

TYPE
  tDataSetAction = (daAppend, daInsert, daEdit, daDelete, daPost, daCancel,
                    daClearFields, daFirst, daNext, daLast, daPrior,
                    daFFirst, daFNext, daFLast, daFPrior, daOpen, daFilterOn );

function ActionDataSet(Script: TIfPasScript; ds : tDataSet; Action : tDataSetAction; CheckModify:Boolean):TIfPasScriptError;
Begin
  ActionDataSet:=ENoError;
  if not (Action in [daOpen,daFilterOn]) and not CheckActiveDataset(Script, ds) then Begin
    ActionDataSet := ECustomError;
    Exit;
  End else if CheckModify and not ds.CanModify Then begin
    Script.RunError2(Script, ECustomError, 'Datenbank kann nicht modifiziert werden');
    ActionDataSet := ECustomError;
    Exit;
  end else
  try
    Case Action of
      daAppend : ds.Append;
      daInsert : ds.Insert;
      daEdit   : ds.Edit;
      daDelete : ds.Delete;
      daPost   : ds.Post;
      daCancel : ds.Cancel;
      daClearFields : ds.ClearFields;
      daFirst  : ds.First;
      daNext   : ds.Next;
      daLast   : ds.Last;
      daPrior  : ds.Prior;
      daFFirst : ds.FindFirst;
      daFNext  : ds.FindNext;
      daFLast  : ds.FindLast;
      daFPrior : ds.FindPrior;
      daOpen   : ds.Open;
      daFilterOn: ds.Filtered:=true;
    End;
  except
    on E: EDatabaseError do Begin
      Script.RunError2(Script, ECustomError, E.Message);
      ActionDataSet:=ECustomError;
    end;
  end;
End;

function LocateLookupDataset(Script: TIfPasScript; Params: PVariableManager; ds:tDataSet; doLookup:Boolean):Boolean;
var L   : TLocateOptions; I: Integer; keys, Results: String;
    A,V : Variant;
Begin
  LocateLookupDataset:=false;
  Keys:=GetString(Vm_Get(Params, 1));
  I:=GetVarLink(Vm_Get(Params, 2))^.VType^.aTypeid;
  if I=CSV_Array Then
    with GetVarLink(Vm_Get(Params, 2))^.CV_ArrItems do
    Begin
      if Count> 1 Then Begin
        A:=VarArrayCreate([0,Count],varVariant);
        for I:=0 to Count-1 do
        Begin
          Script.IfVariantToVariant(GetItem(i), V);
          A[i]:=V;
        end;
      end else if Count=1 Then
        Script.IfVariantToVariant(GetItem(0), A);
    end;
  if doLookup then
    Results:=GetString(Vm_Get(Params, 3))
  else begin
    L:=[]; I:=GetInteger(Vm_Get(Params, 3));
    if (I and 1) <> 0 Then L:=L+[loCaseInsensitive];
    if (I and 2) <> 0 Then L:=L+[loPartialKey];
  end;
  Try
    if doLookup Then
      LocateLookupDataset:=ds.Lookup(Keys,A,Results)
    else LocateLookupDataset:=ds.Locate(Keys,A,L);
  except
    on E: EDatabaseError do Script.RunError2(Script, ECustomError, E.Message);
  end;
end;

TYPE tAsFieldType = (asInteger, asReal, asString);

function GetFieldAs(Script: TIfPasScript; Params: PVariableManager;
                    res: PIfVariant; ds:tDataSet; getAs: tAsFieldType ): Boolean;
Begin
  GetFieldAs:=false;
  try
    Case GetAs of
      asInteger: SetInteger(res,ds.FieldByName(GetString(Vm_Get(Params,1))).AsInteger);
      asReal   : SetReal(res,ds.FieldByName(GetString(Vm_Get(Params,1))).AsFloat);
      asString : SetString(res,ds.FieldByName(GetString(Vm_Get(Params,1))).AsString);
    end;
    GetFieldAs:=true;
  except
    on E: EDatabaseError do Script.RunError2(Script, ECustomError, E.Message);
  end;
end;

function SetFieldAs(Script: TIfPasScript; Params: PVariableManager;
                    ds:tDataSet; SetAs: tAsFieldType) : Boolean;
Begin
  SetFieldAs:=false;
  try
    Case SetAs of
      asInteger: ds.FieldByName(GetString(Vm_Get(Params,1))).AsInteger:=GetInteger(Vm_Get(Params,2));
      asReal   : ds.FieldByName(GetString(Vm_Get(Params,1))).AsFloat:=GetReal(Vm_Get(Params,2));
      asString : ds.FieldByName(GetString(Vm_Get(Params,1))).AsString:=GetString(Vm_Get(Params,2));
    end;
    SetFieldAs:=true;
  except
    on E: EDatabaseError do Script.RunError2(Script, ECustomError, E.Message);
  end;
end;


function TDataConProc(Sender: TIfPasScript; ScriptID: Pointer; proc: PProcedure; Params: PVariableManager; res: PIfVariant): TIfPasScriptError;
var  Ctrl, Self : PIfVariant; ds:tDataSet;
     W:Word; F: tFilterOptions;
begin
  Self := Vm_Get(Params, 0);
  if (not GetClassVariable(Self, PTypeRec(proc^.ClassType)^.Ext, 'FDATACON', Ctrl, True)) or (Ctrl^.Cv_Int1 = nil) then begin
    TDataConProc := ENotSupported; // internal error
    exit;
  end;
  TDataConProc := ENoError;
  ds:=TDataSet(Ctrl^.Cv_Int1);
  if proc^.Name = '!CREATE' then begin
    Sender.AddResource(TDataConFree, Ctrl^.Cv_Int1);
  end else if proc^.Name = '!DESTROY' then begin
    if Sender.IsValidResource(TDataConFree, Ctrl^.Cv_Int1) then begin
      Sender.RemoveResource(Ctrl^.Cv_Int1);
      TDataConFree(ScriptID,Ctrl^.Cv_Int1);
      Ctrl^.Cv_Int1 := nil;
    end else
      TDataConProc := EClassNotCreated; // EInternalError
  end else
  // ----- Propertys ReadOnly ---------------
  if proc^.Name = '!GETBOF' then begin
    if CheckActiveDataset(Sender,ds) then
      SetBoolean(res, ds.BOF)
    else TDataConProc := ECustomError;
  end else if proc^.Name = '!GETEOF' then begin
    if CheckActiveDataset(Sender,ds) then
      SetBoolean(res, ds.EOF)
    else TDataConProc := ECustomError;
  end else if proc^.Name = '!GETCANMODIFY' then begin
    SetBoolean(res, ds.CanModify);
  end else if proc^.Name = '!GETFIELDCOUNT' then begin
    // noch bearbeiten
  end else if proc^.Name = '!GETFOUND' then begin
    if CheckActiveDataset(Sender,ds) then
      SetBoolean(res, ds.Found)
    else TDataConProc := ECustomError;
  end else if proc^.Name = '!GETMODIFIED' then begin
    if CheckActiveDataset(Sender,ds) then
      SetBoolean(res, ds.Modified)
    else TDataConProc := ECustomError;
  end else if proc^.Name = '!GETRECORDCOUNT' then begin
    if CheckActiveDataset(Sender,ds) then
      SetInteger(res, ds.RecordCount)
    else TDataConProc := ECustomError;
  end else if proc^.Name = '!GETRECORDSIZE' then begin
    if CheckActiveDataset(Sender,ds) then
      SetInteger(res, ds.RecordSize)
    else TDataConProc := ECustomError;
  end else if proc^.Name = '!GETSTATE' then begin
    SetInteger(res,Integer(ds.State));
  end else
  // ----- Propertys Read/Write ---------------
  if proc^.Name = '!GETACTIVE' then begin
    SetBoolean(res, ds.Active);
  end else if proc^.Name = '!GETFILTERED' then begin
    SetBoolean(res, ds.Filtered);
  end else if proc^.Name = '!GETFILTEROPTIONS' then begin
    W:=0;
    If (foCaseInsensitive in ds.FilterOptions) Then Inc(w);
    If (foNoPartialCompare in ds.FilterOptions) Then Inc(w,2);
    SetInteger(res,Integer(w));
  end else if proc^.Name = '!GETFILTERTEXT' then begin
    SetString(res, ds.Filter);
  end else if proc^.Name = '!SETACTIVE' then begin
    if GetBoolean(Vm_Get(Params,1))=False Then
      ds.Close
    else TDataConProc := ActionDataSet(Sender, ds, daOpen, false);
  end else if proc^.Name = '!SETFILTERED' then begin
    if GetBoolean(Vm_Get(Params,1))=False Then
      ds.Filtered:=false
    else TDataConProc := ActionDataSet(sender, ds, daFilterOn,false );
  end else if proc^.Name = '!SETFILTEROPTIONS' then begin
    F:=[]; w:=GetInteger(Vm_Get(Params, 1));
    if (w and 1) <> 0 Then F:=F+[foCaseInsensitive];
    if (w and 2) <> 0 Then F:=F+[foNoPartialCompare];
    ds.FilterOptions:=F;
  end else if proc^.Name = '!SETFILTERTEXT' then begin
    try
      ds.Filter:=GetString(Vm_Get(Params,1));
    except
      on E: EDatabaseError do Begin
        sender.RunError2(sender, ECustomError, E.Message);
        TDataConProc := ECustomError;
      end;
    end;
  end else
  // ----- Edit and Update Section ---------------
  if proc^.Name = '!APPEND' then begin
    TDataConProc:=ActionDataSet(Sender,ds,daAppend,true);
  end else if proc^.Name = '!INSERT' then begin
    TDataConProc:=ActionDataSet(Sender,ds,daInsert,true);
  end else if proc^.Name = '!EDIT' then begin
    TDataConProc:=ActionDataSet(Sender,ds,daEdit,true);
  end else if proc^.Name = '!DELETE' then begin
    TDataConProc:=ActionDataSet(Sender,ds,daDelete,true);
  end else if proc^.Name = '!POST' then begin
    TDataConProc:=ActionDataSet(Sender,ds,daPost,true);
  end else if proc^.Name = '!CANCEL' then begin
    TDataConProc:=ActionDataSet(Sender,ds,daCancel,true);
  end else if proc^.Name = '!CLEARFIELDS' then begin
    TDataConProc:=ActionDataSet(Sender,ds,daClearFields,true);
  end else if proc^.Name = '!APPENDRECORD' then begin
   // procedure AppendRecord( Values: array of const )
  end else if proc^.Name = '!INSERTRECORD' then begin
   // procedure InsertRecord( Values: array of const )
  end else if proc^.Name = '!SETFIELDS' then begin
   // procedure SetFields( Values: array of const )
  end else
  // ----- Open Close Section --------------------
  if proc^.Name = '!OPEN' then begin
    TDataConProc := ActionDataSet(Sender, ds, daOpen, false);
  end else if proc^.Name = '!CLOSE' then begin
    if ds.Active then ds.Close;
  end else if proc^.Name = '!REFRESH' then begin
    if ds.Active then ds.Refresh;
  end else
  // ----- Locate Section ------------------------
  if proc^.Name = '!' then begin
  end else if proc^.Name = '!FIRST' then begin
    TDataConProc:=ActionDataSet(Sender,ds,daFirst,false);
  end else if proc^.Name = '!LAST' then begin
    TDataConProc:=ActionDataSet(Sender,ds,daLast,false);
  end else if proc^.Name = '!NEXT' then begin
    TDataConProc:=ActionDataSet(Sender,ds,daNext,false);
  end else if proc^.Name = '!PRIOR' then begin
    TDataConProc:=ActionDataSet(Sender,ds,daPrior,false);
  end else if proc^.Name = '!MOVEBY' then begin
    if CheckActiveDataset(sender,ds) then
      SetInteger(res,ds.MoveBy(GetInteger(Vm_Get(Params, 1))))
    else TDataConProc := ECustomError;
  end else if proc^.Name = '!FINDFIRST' then begin
    TDataConProc:=ActionDataSet(Sender,ds,daFFirst,false);
  end else if proc^.Name = '!FINDLAST' then begin
    TDataConProc:=ActionDataSet(Sender,ds,daFLast,false);
  end else if proc^.Name = '!FINDNEXT' then begin
    TDataConProc:=ActionDataSet(Sender,ds,daFNext,false);
  end else if proc^.Name = '!FINDPRIOR' then begin
    TDataConProc:=ActionDataSet(Sender,ds,daFPrior,false);
  end else if proc^.Name = '!LOCATE' then begin
    if CheckActiveDataset(sender,ds) then
      SetBoolean(res, LocateLookupDataset(sender, params, ds, false))
    else TDataConProc := ECustomError;
  end else if proc^.Name = '!LOOKUP' then begin
    if CheckActiveDataset(sender,ds) then
      SetBoolean(res, LocateLookupDataset(sender, params, ds, true))
    else TDataConProc := ECustomError;
  end else if proc^.Name = '!GETBOOKMARK' then begin
    if CheckActiveDataset(Sender,ds) then
      SetInteger(Res, Integer(ds.GetBookmark))
    else TDataConProc := ECustomError;
  end else if proc^.Name = '!GOTOBOOKMARK' then begin
    if CheckActiveDataset(Sender,ds) then
      ds.GotoBookmark(Pointer(GetInteger(Vm_Get(Params,1))))
    else TDataConProc := ECustomError;
  end else if proc^.Name = '!FREEBOOKMARK' then begin
    if CheckActiveDataset(Sender,ds) then
      ds.FreeBookmark(Pointer(GetInteger(Vm_Get(Params,1))))
    else TDataConProc := ECustomError;
  end else if proc^.Name = '!BOOKMARKVALID' then begin
    if CheckActiveDataset(Sender,ds) then
      SetBoolean(res,  ds.BookmarkValid(Pointer(GetInteger(Vm_Get(Params,1)))))
    else TDataConProc := ECustomError;
  end else if proc^.Name = '!GETFIELD' then begin
    if not CheckActiveDataset(sender,ds) Then
      TDataConProc:=ECustomError
    else if not GetFieldAs(Sender,params,res,ds, asString) Then
      TDataConProc:=ECustomError
  end else if proc^.Name = '!SETFIELD' then begin
    if not CheckActiveDataset(sender,ds) Then
      TDataConProc:=ECustomError
    else if not SetFieldAs(Sender,params,ds, asString) Then
      TDataConProc:=ECustomError;
  end else if proc^.Name = '!GETFIELDAS' then begin
    if not CheckActiveDataset(sender,ds) Then
      TDataConProc:=ECustomError
    else if not GetFieldAs(Sender,params,res,ds, tAsFieldType(GetInteger(Vm_Get(Params,3)))) Then
      TDataConProc:=ECustomError;
  end else if proc^.Name = '!SETFIELDAS' then begin
    if not CheckActiveDataset(sender,ds) Then
      TDataConProc:=ECustomError
    else if not SetFieldAs(Sender,params,ds, tAsFieldType(GetInteger(Vm_Get(Params,3)))) Then
      TDataConProc:=ECustomError;
  end else
  // ----- Status and Information Section ------------------------
  if proc^.Name = '!' then begin
  end else if proc^.Name = '!ISEMPTY' then begin
    SetBoolean(res, ds.IsEmpty);
  end else if proc^.Name = '!ISSEQUENCED' then begin
    SetBoolean(res, ds.IsSequenced);
  end;
end;

procedure RegisterDBLibrary(p: TIfPasScript);
Begin
  p.AddVariable('loCaseInsensitive', 'Integer', True)^.Cv_SInt32 := 1;
  p.AddVariable('loPartialKey', 'Integer', True)^.Cv_SInt32 := 2;
  p.AddVariable('dsInactive', 'Integer', True)^.Cv_SInt32 := 0;
  p.AddVariable('dsBrowse', 'Integer', True)^.Cv_SInt32 := 1;
  p.AddVariable('dsEdit', 'Integer', True)^.Cv_SInt32 := 2;
  p.AddVariable('dsInsert', 'Integer', True)^.Cv_SInt32 := 3;
  p.AddVariable('dsSetKey', 'Integer', True)^.Cv_SInt32 := 4;
  p.AddVariable('dsCalcFields', 'Integer', True)^.Cv_SInt32 := 5;
  p.AddVariable('dsFilter', 'Integer', True)^.Cv_SInt32 := 6;
  p.AddVariable('asInteger','Integer', True)^.Cv_SInt32 := 0;
  p.AddVariable('asReal','Integer', True)^.Cv_SInt32 := 1;
  p.AddVariable('asString','Integer', True)^.Cv_SInt32 := 2;
  p.AddType('TBookmark', 'Integer;');
  p.AddType('TKeyValues', 'array of const;');
  p.AddClass('TDataCon', TDataConClass, @TDataConProc);
end;

end.

