unit ifsdbado;
{
  Innerfuse Pascal Script ADO Database Library
  Made by Xaver Bauer <xb@condat.com>
}

interface
uses ifspas,ifs_var, ifsdb, ADOdb;

procedure RegisterDBadoLibrary(p: TIfPasScript);

implementation

CONST
  TADOTableClass =
    'class(TDataCon)'+
    'private '+
    'function GetConnectionString : String;'+
    'procedure SetConnectionString( S: String );'+
    'function GetCacheSize: Integer;'+
    'procedure SetCacheSize( I: Integer );'+
    'function GetTablename: String;'+
    'procedure SetTablename( S: String );'+
    'function GetIndexName: String;'+
    'procedure SetIndexName( S: String );'+
    'public '+
    'constructor Create;'+
    'property CacheSize: Integer read GetCacheSize write SetCacheSize;'+
    'property ConnectionString: String read GetConnectionString write SetConnectionString;'+
    'property Tablename: String read GetTablename write SetTablename;'+
    'property IndexName: String read GetIndexName write SetIndexName;'+
    'end;';

  TADOQueryClass =
    'class(TDataCon)'+
    'private '+
    'function GetConnectionString : String;'+
    'procedure SetConnectionString( S: String );'+
    'function GetCacheSize: Integer;'+
    'procedure SetCacheSize( I: Integer );'+
    'function GetSQL: String;'+
    'procedure SetSQL( S: String );'+
    'function GetMaxRecords: Integer;'+
    'procedure SetMaxRecords( I: Integer );'+
    'public '+
    'constructor Create;'+
    'property CacheSize: Integer read GetCacheSize write SetCacheSize;'+
    'property ConnectionString: String read GetConnectionString write SetConnectionString;'+
    'property SQL: String read GetSQL write SetSQL;'+
    'property MaxRecords: Integer read GetMaxRecords write SetMaxRecords;'+
    'end;';

function TADOTableProc(Sender: TIfPasScript; ScriptID: Pointer; proc: PProcedure; Params: PVariableManager; res: PIfVariant): TIfPasScriptError;
var  Ctrl, Self: PIfVariant; ds: tADOTable;
     vm        : PVariableManager;
begin
  Self := Vm_Get(Params, 0);
  if (not GetClassVariable(Self, proc^.ClassType^.Ext, 'FDATACON', Ctrl, True)) then begin
    TADOTableProc := ENotSupported; // internal error
    exit;
  end;
  TADOTableProc := ENoError;
  ds:=tADOTable(Ctrl^.Cv_Int1);
  if proc^.Name = '!CREATE' then begin
    Ctrl^.Cv_Int1 := TADOTable.Create(nil);
    vm := VM_Create(nil);
    Vm_Add(vm, Self, 'SELF');
    Sender.RunScriptProc(GetInheritedProc(proc), vm);
    VM_Delete(vm, 0);
    VM_Destroy(vm);
  end else if proc^.Name = '!GETTABLENAME' then begin
    SetString(res,ds.TableName);
  end else if proc^.Name = '!SETTABLENAME' then begin
    if ds.Active then ds.Close;
    ds.TableName:=GetString(Vm_Get(params,1));
  end else if proc^.Name = '!GETINDEXNAME' then begin
    SetString(res,ds.IndexName);
  end else if proc^.Name = '!SETINDEXNAME' then begin
    if ds.Active then ds.Close;
    ds.IndexName:=GetString(Vm_Get(params,1));
  end else if proc^.Name = '!GETREADONLY' then begin
    SetBoolean(res, ds.ReadOnly);
  end else if proc^.Name = '!SETREADONLY' then begin
    if ds.Active then ds.Close;
    ds.ReadOnly:=GetBoolean(Vm_Get(params,1));
  end else if proc^.Name = '!GETCONNECTIONSTRING' then begin
    SetString(res, ds.ConnectionString);
  end else if proc^.Name = '!SETCONNECTIONSTRING' then begin
    if ds.Active then ds.Close;
    ds.ConnectionString:=GetString(Vm_Get(params,1));
  end else if proc^.Name = '!SETCACHESIZE' then begin
    if ds.Active then ds.Close;
    ds.CacheSize:=GetInteger(Vm_Get(params,1));
  end else if proc^.Name = '!GETCACHESIZE' then begin
    SetInteger(res,ds.CacheSize);
  end;
end;

function TADOQueryProc(Sender: TIfPasScript; ScriptID: Pointer; proc: PProcedure; Params: PVariableManager; res: PIfVariant): TIfPasScriptError;
var  Ctrl, Self: PIfVariant; ds: tADOQuery;
     vm        : PVariableManager;
begin
  Self := Vm_Get(Params, 0);
  if (not GetClassVariable(Self, proc^.ClassType^.Ext, 'FDATACON', Ctrl, True)) then begin
    TADOQueryProc := ENotSupported; // internal error
    exit;
  end;
  TADOQueryProc := ENoError;
  ds:=tADOQuery(Ctrl^.Cv_Int1);
  if proc^.Name = '!CREATE' then begin
    Ctrl^.Cv_Int1 := TADOQuery.Create(nil);
    vm := VM_Create(nil);
    Vm_Add(vm, Self, 'SELF');
    Sender.RunScriptProc(GetInheritedProc(proc), vm);
    VM_Delete(vm, 0);
    VM_Destroy(vm);
  end else if proc^.Name = '!GETSQL' then begin
    SetString(res,ds.SQL.Text);
  end else if proc^.Name = '!SETSQL' then begin
    if ds.Active then ds.Close;
    ds.SQL.Text:=GetString(Vm_Get(params,1));
  end else if proc^.Name = '!GETMAXRECORDS' then begin
    SetInteger(res, ds.MaxRecords);
  end else if proc^.Name = '!SETMAXRECORDS' then begin
    if ds.Active then ds.Close;
    ds.MaxRecords:=GetInteger(Vm_Get(params,1));
  end else if proc^.Name = '!GETCONNECTIONSTRING' then begin
    SetString(res, ds.ConnectionString);
  end else if proc^.Name = '!SETCONNECTIONSTRING' then begin
    if ds.Active then ds.Close;
    ds.ConnectionString:=GetString(Vm_Get(params,1));
  end else if proc^.Name = '!SETCACHESIZE' then begin
    if ds.Active then ds.Close;
    ds.CacheSize:=GetInteger(Vm_Get(params,1));
  end else if proc^.Name = '!GETCACHESIZE' then begin
    SetInteger(res,ds.CacheSize);
  end;

end;

procedure RegisterDBadoLibrary(p: TIfPasScript);
Begin
  p.AddClass('TADOTable', TADOTableClass, @TADOTableProc);
  p.AddClass('TADOQuery', TADOQueryClass, @TADOQueryProc);
end;

end.
