unit ifsdbbde;
{
  Innerfuse Pascal Script BDE Database Library
  Made by Xaver Bauer <xb@condat.com>
}

interface
uses ifspas,ifs_var, ifsdb, DBTables;

procedure RegisterDBbdeLibrary(p: TIfPasScript);

implementation

CONST
  TTableClass =
    'class(TDataCon)'+
    'private '+
    'function GetDatabase : String;'+
    'procedure SetDatabase( S: String );'+
    'function GetCacheUpdates: Boolean;'+
    'procedure SetCacheUpdates( B: Boolean );'+
    'function GetTablename: String;'+
    'procedure SetTablename( S: String );'+
    'function GetReadOnly: Boolean;'+
    'procedure SetReadOnly( B: Boolean );'+
    'function GetIndexName: String;'+
    'procedure SetIndexName( S: String );'+
    'public '+
    'constructor Create;'+
    'property CacheUpdates: Boolean read GetCacheUpdates write SetCacheUpdates;'+
    'property Database: String read GetDatabase write SetDatabase;'+
    'property Tablename: String read GetTablename write SetTablename;'+
    'property ReadOnly: Boolean read GetReadOnly write SetReadOnly;'+
    'property IndexName: String read GetIndexName write SetIndexName;'+
    'end;';

  TQueryClass =
    'class(TDataCon)'+
    'private '+
    'function GetDatabase : String;'+
    'procedure SetDatabase( S: String );'+
    'function GetCacheUpdates: Boolean;'+
    'procedure SetCacheUpdates( B: Boolean );'+
    'function GetSQL: String;'+
    'procedure SetSQL( S: String );'+
    'function GetReadOnly : Boolean;'+
    'procedure SetReadOnly( B: Boolean );'+
    'public '+
    'constructor Create;'+
    'property CacheUpdates: Boolean read GetCacheUpdates write SetCacheUpdates;'+
    'property Database: String read GetDatabase write SetDatabase;'+
    'property SQL: String read GetSQL write SetSQL;'+
    'property ReadOnly: Boolean read GetReadOnly write SetReadOnly;'+
    'end;';

function TTableProc(Sender: TIfPasScript; ScriptID: Pointer; proc: PProcedure; Params: PVariableManager; res: PIfVariant): TIfPasScriptError;
var  Ctrl, Self: PIfVariant; ds: tTable;
     vm        : PVariableManager;
begin
  Self := Vm_Get(Params, 0);
  if (not GetClassVariable(Self, proc^.ClassType^.Ext, 'FDATACON', Ctrl, True)) then begin
    TTableProc := ENotSupported; // internal error
    exit;
  end;
  TTableProc := ENoError;
  ds:=tTable(Ctrl^.Cv_Int1);
  if proc^.Name = '!CREATE' then begin
    Ctrl^.Cv_Int1 := TTable.Create(nil);
    vm := VM_Create(nil);
    Vm_Add(vm, Self, 'SELF');
    Sender.RunScriptProc(GetInheritedProc(proc), vm);
    VM_Delete(vm, 0);
    VM_Destroy(vm);
  end else if proc^.Name = '!GETTABLENAME' then begin
    SetString(res,ds.TableName);
  end else if proc^.Name = '!SETTABLENAME' then begin
    if ds.Active then ds.Close;
    ds.TableName:=GetString(Vm_Get(params,1));
  end else if proc^.Name = '!GETINDEXNAME' then begin
    SetString(res,ds.IndexName);
  end else if proc^.Name = '!SETINDEXNAME' then begin
    if ds.Active then ds.Close;
    ds.IndexName:=GetString(Vm_Get(params,1));
  end else if proc^.Name = '!GETREADONLY' then begin
    SetBoolean(res, ds.ReadOnly);
  end else if proc^.Name = '!SETREADONLY' then begin
    if ds.Active then ds.Close;
    ds.ReadOnly:=GetBoolean(Vm_Get(params,1));
  end else if proc^.Name = '!GETDATABASE' then begin
    SetString(res, ds.DatabaseName);
  end else if proc^.Name = '!SETDATABASE' then begin
    if ds.Active then ds.Close;
    ds.DatabaseName:=GetString(Vm_Get(params,1));
  end else if proc^.Name = '!SETCACHEUPDATES' then begin
    if ds.Active then ds.Close;
    ds.CachedUpdates:=GetBoolean(Vm_Get(params,1));
  end else if proc^.Name = '!GETCACHEUPDATES' then begin
    SetBoolean(res,ds.CachedUpdates);
  end;
end;

function TQueryProc(Sender: TIfPasScript; ScriptID: Pointer; proc: PProcedure; Params: PVariableManager; res: PIfVariant): TIfPasScriptError;
var  Ctrl, Self: PIfVariant; ds : tQuery;
     vm        : PVariableManager;
begin
  Self := Vm_Get(Params, 0);
  if (not GetClassVariable(Self, proc^.ClassType^.Ext, 'FDATACON', Ctrl, True)) then begin
    TQueryProc := ENotSupported; // internal error
    exit;
  end;
  TQueryProc := ENoError;
  ds:=tQuery(Ctrl^.Cv_Int1);
  if proc^.Name = '!CREATE' then begin
    Ctrl^.Cv_Int1 := TQuery.Create(nil);
    vm := VM_Create(nil);
    Vm_Add(vm, Self, 'SELF');
    Sender.RunScriptProc(GetInheritedProc(proc), vm);
    VM_Delete(vm, 0);
    VM_Destroy(vm);
  end else if proc^.Name = '!GETSQL' then begin
    SetString(res,ds.SQL.Text);
  end else if proc^.Name = '!SETSQL' then begin
    if ds.Active then ds.Close;
    ds.SQL.Text:=GetString(Vm_Get(params,1));
  end else if proc^.Name = '!GETREADONLY' then begin
    SetBoolean(res, ds.RequestLive);
  end else if proc^.Name = '!SETREADONLY' then begin
    if ds.Active then ds.Close;
    ds.RequestLive:=GetBoolean(Vm_Get(params,1));
  end else if proc^.Name = '!GETDATABASE' then begin
    SetString(res, ds.DatabaseName);
  end else if proc^.Name = '!SETDATABASE' then begin
    if ds.Active then ds.Close;
    ds.DatabaseName:=GetString(Vm_Get(params,1));
  end else if proc^.Name = '!SETCACHEUPDATES' then begin
    if ds.Active then ds.Close;
    ds.CachedUpdates:=GetBoolean(Vm_Get(params,1));
  end else if proc^.Name = '!GETCACHEUPDATES' then begin
    SetBoolean(res,ds.CachedUpdates);
  end
end;

procedure RegisterDBbdeLibrary(p: TIfPasScript);
Begin
  p.AddClass('TTable', TTableClass, @TTableProc);
  p.AddClass('TQuery', TQueryClass, @TQueryProc);
end;

end.
