unit streamlb;
{
  Innerfuse Pascal Script Library
  For license see Innerfuse Pascal Script license file

  Library Written By: Adel Samuel
  E-mail At: adelsamuel@sympatico.ca
  Version 0.8 (Updated Test Version)
}
{$I LB_DEF.INC}
interface
uses
  ifspas, ifs_var, Classes, SysUtils {IFNDEF NOPOINTERS} , TypeCast {$ENDIF} ;

procedure RegisterStreams(p: TIfPasScript);

implementation

procedure TStreamFree(id: Pointer; Data: Pointer);
begin
  TStream(Data).Free;
end;

const

  TStreamT =
    'class(TObject)' +
    'private ' +
    'function GetPosition: Integer;' +
    'procedure SetPosition(L: Integer);' +
    'function GetSize: Integer;' +
    'procedure SetSize(L:Integer);' +
    'protected ' +
    'Data: ResourcePointer;' +
    'public ' +
    'constructor Create;' +
    'destructor Destroy; override;' +
    'function CopyFrom(Source: TStream; Count: Integer): Integer;' +
    'property Position: Integer read GetPosition write SetPosition;' +
    'property Size: Integer read GetSize write SetSize;' +
{$IFNDEF NOPOINTERS}
  'function Read(var s: Pointer; Count: Integer): Integer; virtual;' +
    'function Write(var s: Pointer; Count: Integer): Integer; virtual;' +
{$ELSE}
  'function Read(var s: string; Count: Integer): Integer; virtual;' +
    'function Write(var s: string; Count: Integer): Integer; virtual;' +
{$ENDIF}
  'function Seek(Offset: Integer; Origin: Word): Integer; virtual;' +
    'end;';

  TMemoryStreamT =
    'class(TStream)' +
    'public ' +
    'constructor Create;' +
    'procedure Clear;' +
{$IFNDEF NOFILEACCESS}
  'procedure LoadFromFile(FileName: string);' +
    'procedure SaveToFile(FileName: string);' +
{$ENDIF}
  'procedure LoadFromStream(Stream: TStream);' +
    'procedure SaveToStream(Stream: TStream);' +
{$IFNDEF NOPOINTERS}
  'function Read(var s: Pointer; Count: Integer): Integer; override;' +
    'function Write(var s: Pointer; Count: Integer): Integer; override;' +
{$ELSE}
  'function Read(var s: string; Count: Integer): Integer; override;' +
    'function Write(var s: string; Count: Integer): Integer; override;' +
{$ENDIF}
  'function Seek(Offset: Integer; Origin: Word): Integer; override;' +
    'end;';

function StreamProc(Sender: TIfPasScript; ScriptID: Pointer; proc: PProcedure;
  Params: PVariableManager; res: PIfVariant): TIfPasScriptError;
var
  p, Self, Stream: PIfVariant;
begin
  Self := Vm_Get(Params, 0);
  StreamProc := ENoError;
  if not GetClassVariable2(Self^.CV_Class, proc^.ClassType^.Ext, 'DATA', p, True)
    then
  begin
    StreamProc := ENotSupported;
    exit;
  end;
  if proc^.Name = '!CREATE' then
  begin
{$WARNINGS OFF}
    p^.Cv_Int1 := TStream.Create;
    Sender.AddResource(TStreamFree, p^.Cv_Int1);
{$WARNINGS ON}
  end
  else
    if proc^.Name = '!DESTROY' then
  begin
    if Sender.IsValidResource(TStreamFree, p^.Cv_Int1) then
    begin
      Sender.RemoveResource(p^.Cv_Int1);
      TStream(p^.Cv_Int1).Free;
      p^.Cv_Int1 := nil;
    end
    else
      StreamProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!READ' then
  begin
    if Sender.IsValidResource(TStreamFree, p^.Cv_Int1) then
    begin
      try
{$IFNDEF NOPOINTERS}
        SetInteger(Res, TStream(p^.Cv_Int1).Read(GetVarLink(Vm_Get(Params,
          1)).CV_Int1,
          GetInteger(GetVarLink(Vm_Get(Params, 2)))));
{$ELSE}
        SetInteger(Res, TStream(p^.Cv_Int1).Read(GetVarLink(Vm_Get(Params,
          1)).CV_Str,
          GetInteger(GetVarLink(Vm_Get(Params, 2)))));
{$ENDIF}
      except
        on e: Exception do
        begin
          StreamProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StreamProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!WRITE' then
  begin
    if Sender.IsValidResource(TStreamFree, p^.Cv_Int1) then
    begin
      try
{$IFNDEF NOPOINTERS}
        SetInteger(Res, TStream(p^.Cv_Int1).Write(GetVarLink(Vm_Get(Params,
          1)).CV_Int1,
          GetInteger(GetVarLink(Vm_Get(Params, 2)))));
{$ELSE}
        SetInteger(Res, TStream(p^.Cv_Int1).Write(GetVarLink(Vm_Get(Params,
          1)).CV_Str,
          GetInteger(GetVarLink(Vm_Get(Params, 2)))));
{$ENDIF}
      except
        on e: Exception do
        begin
          StreamProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
  end
  else
    if proc^.Name = '!SEEK' then
  begin
    if Sender.IsValidResource(TStreamFree, p^.Cv_Int1) then
    begin
      try
        SetInteger(Res,
          TStream(p^.Cv_Int1).Seek(GetInteger(GetVarLink(Vm_Get(Params,
          1))),
          GetInteger(GetVarLink(Vm_Get(Params, 2)))));
      except
        on e: Exception do
        begin
          StreamProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StreamProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!GETPOSITION' then
  begin
    if Sender.IsValidResource(TStreamFree, p^.Cv_Int1) then
    begin
      try
        SetInteger(Res,
          TStream(p^.Cv_Int1).Position);
      except
        on e: Exception do
        begin
          StreamProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StreamProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!SETPOSITION' then
  begin
    if Sender.IsValidResource(TStreamFree, p^.Cv_Int1) then
    begin
      try
        TStream(p^.Cv_Int1).Position :=
          GetInteger(GetVarLink(Vm_Get(Params,
          1)));
      except
        on e: Exception do
        begin
          StreamProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StreamProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!SETSIZE' then
  begin
    if Sender.IsValidResource(TStreamFree, p^.Cv_Int1) then
    begin
      TStream(p^.Cv_Int1).Size := GetInteger(GetVarLink(Vm_Get(Params,
        1)));
    end
    else
      StreamProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!GETSIZE' then
  begin
    if Sender.IsValidResource(TStreamFree, p^.Cv_Int1) then
    begin
      SetInteger(Res,
        TStream(p^.Cv_Int1).Size);
    end
    else
      StreamProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!COPYFROM' then
  begin
    if Sender.IsValidResource(TStreamFree, p^.Cv_Int1) then
    begin
      if (not GetClassVariable(Vm_Get(Params, 1),
        Sender.GetType('TSTREAM')^.Ext,
        'DATA', Stream, True)) or (Stream^.Cv_Int1 = nil) then
      begin
        StreamProc := ENotSupported;
        Exit;
      end;
      if not (TOBJECT(Stream^.CV_Int1) is TStream) then
      begin
        StreamProc := ENotSupported;
        Exit;
      end;
      try
        SetInteger(Res,
          TStream(p^.Cv_Int1).CopyFrom(TStream(Stream^.Cv_Int1),
          GetInteger(GetVarLink(VM_GET(PARAMS, 2)))));
      except
        on e: Exception do
        begin
          StreamProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StreamProc := EClassNotCreated; // EInternalError
  end
  else
    StreamProc := EUnknownIdentifier;
end;

function MemoryStreamProc(Sender: TIfPasScript; ScriptID: Pointer; proc:
  PProcedure;
  Params: PVariableManager; res: PIfVariant): TIfPasScriptError;
var
  p, Self, Stream: PIfVariant;
begin
  Self := Vm_Get(Params, 0);
  MemoryStreamProc := ENoError;
  if not GetClassVariable2(Self^.CV_Class, proc^.ClassType^.Ext, 'DATA', p, True)
    then
  begin
    MemoryStreamProc := ENotSupported;
    exit;
  end;
  if proc^.Name = '!CREATE' then
  begin
    p^.Cv_Int1 := TMemoryStream.Create;
    Sender.AddResource(TStreamFree, p^.Cv_Int1);
  end
  else
    if proc^.Name = '!READ' then
  begin
    if Sender.IsValidResource(TStreamFree, p^.Cv_Int1) then
    begin
      try
{$IFNDEF NOPOINTERS}
        SetInteger(Res, TMemoryStream(p^.Cv_Int1).Read(GetVarLink(Vm_Get(Params,
          1)).CV_Int1,
          GetInteger(GetVarLink(Vm_Get(Params, 2)))));
{$ELSE}
        SetInteger(Res, TMemoryStream(p^.Cv_Int1).Read(GetVarLink(Vm_Get(Params,
          1)).CV_Str,
          GetInteger(GetVarLink(Vm_Get(Params, 2)))));
{$ENDIF}
      except
        on e: Exception do
        begin
          MemoryStreamProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      MemoryStreamProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!WRITE' then
  begin
    if Sender.IsValidResource(TStreamFree, p^.Cv_Int1) then
    begin
      try
{$IFNDEF NOPOINTERS}
        SetInteger(Res,
          TMemoryStream(p^.Cv_Int1).Write(GetVarLink(Vm_Get(Params,
          1)).CV_Int1,
          GetInteger(GetVarLink(Vm_Get(Params, 2)))));
{$ELSE}
        SetInteger(Res,
          TMemoryStream(p^.Cv_Int1).Write(GetVarLink(Vm_Get(Params,
          1)).CV_Str,
          GetInteger(GetVarLink(Vm_Get(Params, 2)))));
{$ENDIF}
      except
        on e: Exception do
        begin
          MemoryStreamProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
  end
  else
    if proc^.Name = '!SEEK' then
  begin
    if Sender.IsValidResource(TStreamFree, p^.Cv_Int1) then
    begin
      try
        SetInteger(Res,
          TMemoryStream(p^.Cv_Int1).Seek(GetInteger(GetVarLink(Vm_Get(Params,
          1))),
          GetInteger(GetVarLink(Vm_Get(Params, 2)))));
      except
        on e: Exception do
        begin
          MemoryStreamProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      MemoryStreamProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!CLEAR' then
  begin
    if Sender.IsValidResource(TStreamFree, p^.Cv_Int1) then
    begin
      try
        TMemoryStream(p^.Cv_Int1).Clear;
      except
        on E: Exception do
        begin
          MemoryStreamProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      MemoryStreamProc := EClassNotCreated; // EInternalError
  end

  else
{$IFNDEF NOFILEACCESS}
    if proc^.Name = '!LOADFROMFILE' then
  begin
    if Sender.IsValidResource(TStreamFree, p^.Cv_Int1) then
    begin
      try
        TMemoryStream(p^.Cv_Int1).LoadFromFile(GetString(GetVarLink(Vm_Get(Params,
          1))));
      except
        on e: Exception do
        begin
          MemoryStreamProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      MemoryStreamProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!SAVETOFILE' then
  begin
    if Sender.IsValidResource(TStreamFree, p^.Cv_Int1) then
    begin
      try
        TMemoryStream(p^.Cv_Int1).SaveToFile(GetString(GetVarLink(Vm_Get(Params,
          1))));
      except
        on e: Exception do
        begin
          MemoryStreamProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      MemoryStreamProc := EClassNotCreated; // EInternalError
  end
  else
{$ENDIF}
    if proc^.Name = '!LOADFROMSTREAM' then
  begin
    if Sender.IsValidResource(TStreamFree, p^.Cv_Int1) then
    begin
      if (not GetClassVariable(Vm_Get(Params, 1),
        Sender.GetType('TSTREAM')^.Ext,
        'DATA', Stream, True)) or (Stream^.Cv_Int1 = nil) then
      begin
        MemoryStreamProc := ENotSupported;
        Exit;
      end;
      if not (TOBJECT(Stream^.CV_Int1) is TStream) then
      begin
        MemoryStreamProc := ENotSupported;
        Exit;
      end;
      try
        TMemoryStream(p^.Cv_Int1).LoadFromStream(TStream(Stream^.CV_Int1));
      except
        on e: Exception do
        begin
          MemoryStreamProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      MemoryStreamProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!SAVETOSTREAM' then
  begin
    if Sender.IsValidResource(TStreamFree, p^.Cv_Int1) then
    begin
      if (not GetClassVariable(Vm_Get(Params, 1),
        Sender.GetType('TSTREAM')^.Ext,
        'DATA', Stream, True)) or (Stream^.Cv_Int1 = nil) then
      begin
        MemoryStreamProc := ENotSupported;
        Exit;
      end;
      if not (TOBJECT(Stream^.CV_Int1) is TStream) then
      begin
        MemoryStreamProc := ENotSupported;
        Exit;
      end;
      try
        TMemoryStream(p^.Cv_Int1).SaveToStream(TStream(Stream^.CV_Int1));
      except
        on e: Exception do
        begin
          MemoryStreamProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      MemoryStreamProc := EClassNotCreated; // EInternalError
  end
  else
    MemoryStreamProc := EUnknownIdentifier;
end;

procedure RegisterStreams(p: TIfPasScript);
begin
{$IFNDEF NOPOINTERS}
  AddTypeCasting(p);
{$ENDIF}
  p.AddClass('TStream', TStreamT, @StreamProc);
  p.AddClass('TMemoryStream', TMemoryStreamT, @MemoryStreamProc);
end;

end.

