unit strlstlb;
{
  Innerfuse Pascal Script Library
  For license see Innerfuse Pascal Script license file

  Library Written By: Adel Samuel
  E-mail At: adelsamuel@sympatico.ca
  Version 0.8 (Updated Test Version)
}
{$I LB_DEF.INC}
interface
uses
  ifspas, ifs_var, Classes, SysUtils{$IFNDEF NOSTREAMS}, Streamlb{$ENDIF};

type
  TStringListHack = class(TStringList)
  public
    Tag: Pointer;
    IFS: TIfPasScript;
  end;

  TEventsClass = class
    C: TIfPasScript;
    procedure OnChange(Sender: TObject);
    procedure OnChanging(Sender: TObject);
  end;

procedure FreeEventsClasses(Sender: TIfPasScript);

function NewEventsClass(Sender: TIfPasScript): TEventsClass;

procedure RegisterTStringList(p: TIfPasScript);

implementation

var
  EventsClasses: TList;

procedure FreeEventsClasses(Sender: TIfPasScript);
var
  I: Integer;
begin
  for I := EventsClasses.Count - 1 downto 0 do
  begin
    if (Sender = nil) or (TEventsClass(EventsClasses[I]).C = Sender) then
    begin
      TEventsClass(EventsClasses[I]).Free;
      EventsClasses.Delete(I);
    end;
  end;
end;

function NewEventsClass(Sender: TIfPasScript): TEventsClass;
var
  I: Integer;
begin
  for I := 0 to EventsClasses.Count - 1 do
  begin
    if TEventsClass(EventsClasses[I]).C = Sender then
    begin
      Result := EventsClasses[I];
      exit;
    end;
  end;
  Result := TEventsClass.Create;
  Result.C := Sender;
  EventsClasses.Add(Result);
end;

procedure TStrListFree(id: Pointer; Data: Pointer);
begin
  TStringListHack(Data).Free;
end;

const

  TStringListT =
    'class(TObject)' +
    'private ' +
    'data: ResourcePointer;' +
    'FCustomSort: TStringListSortCompare;' +
    'FOnChange, FOnChanging: TNotifyEvent;' +
    'function GetText: string;' +
    'function GetSorted: Boolean;' +
    'function GetCommaText: string;' +
    'function GetDuplicates: Integer;' +
    'procedure SetDuplicates(I: Integer);' +
    'procedure SetText(s: string);' +
    'procedure SetSorted(b: Boolean);' +
    'procedure SetCommaText(s: string);' +
    'public ' +
    'constructor Create;' +
    'destructor Destroy; override;' +
    'procedure Clear;' +
    'procedure Exchange(Index1, Index2: Integer);' +
    'function Find(s: string; var Index: Integer): Boolean;' +
    'function IndexOf(s: string): Integer;' +
    'function IndexOfName(s: string): Integer;' +
    'procedure Insert(Index: Integer; s: string);' +
    'procedure Move(CurIndex, NewIndex: Integer);' +
    'procedure Assign(Source: TStringList);' +
    'function GetItem(I: Integer): string;' +
    'procedure SetItem(I: Integer; S: string);' +
    'function GetObject(I: Integer): TObject;' +
    'procedure SetObject(I: Integer; O: TObject);' +
    'function Count: Integer;' +
    'function Add(s: string): Integer;' +
    'procedure Append(s: string);' +
    'function GetValues(Name: string): string;' +
    'procedure SetValues(Name: string; Value: string);' +
    'function AddObject(s: string; AObject: TObject): Integer;' +
    'procedure InsertObject(Index: Integer; S: string; AObject: TObject);' +
    'function IndexOfObject(AObject: TObject): Integer;' +
    'function Equals(Strings: TStringList): Boolean;' +
    'function Names(Index: Integer): string;' +
    'procedure Delete(I: Integer);' +
{$IFNDEF NOFILEACCESS}
  'procedure LoadFromFile(FileName: string);' +
    'procedure SaveToFile(FileName: string);' +
{$ENDIF}
{$IFNDEF NOSTREAMS}
  'procedure LoadFromStream(Stream: TStream);' +
    'procedure SaveToStream(Stream: TStream);' +
{$ENDIF}
  'procedure Sort;' +
    'procedure AddStrings(Strings: TStringList);' +
    'procedure CustomSort(Compare: TStringListSortCompare);' +
    'property Sorted: Boolean read GetSorted write SetSorted;' +
    'property CommaText: string read GetCommaText write SetCommaText;' +
    'property Text: string read GetText write SetText;' +
    'property Duplicates: Integer read GetDuplicates write SetDuplicates;' +
    'property OnChange: TNotifyEvent read FOnChange write FOnChange;' +
    'property OnChanging: TNotifyEvent read FOnChanging write FOnChanging;' +
    'end;';

function StrCustomSort(List: TStringList; Index1, Index2:
  Integer): Integer;
var
  p: PCreatedClass;
  Vars: PVariableManager;
  E, CV, Indexv1, Indexv2: PIfVariant;
begin
  Result := 0;
  p := Pointer(TStringListHack(List).Tag);
  if not assigned(p) then exit;
  if not GetClassVariable2(p, p^.ClassType^.Ext, 'FCUSTOMSORT', E, True) then
    exit;
  if not assigned(E^.Cv_Proc) then exit;
  Vars := VM_Create(nil);
  try
    CV := CreateCajVariant(p^.ClassType);
    Vm_Add(Vars, CV, 'List')^.CV_Class := p;
    Indexv1 := CreateCajVariant(TStringListHack(List).IFS.GetType('INTEGER'));
    Indexv2 := CreateCajVariant(TStringListHack(List).IFS.GetType('INTEGER'));
    VM_Add(Vars, Indexv1, 'Index1')^.CV_SInt32 := Index1;
    VM_Add(Vars, Indexv2, 'Index2')^.CV_SInt32 := Index2;
    Result := GetInteger(TStringListHack(List).IFS.RunScriptProc(E^.Cv_Proc,
      Vars));
  finally
    VM_Destroy(Vars);
  end;
end;

procedure TEventsClass.OnChange(Sender: TObject);
var
  p: PCreatedClass;
  Vars: PVariableManager;
  E, CV: PIfVariant;
begin
  p := Pointer(TStringListHack(Sender).Tag);
  if not assigned(p) then exit;
  if not GetClassVariable2(p, p^.ClassType^.Ext, 'FONCHANGE', E, True) then
    exit;
  if not assigned(E^.Cv_Proc) then exit;
  Vars := VM_Create(nil);
  try
    CV := CreateCajVariant(p^.ClassType);
    Vm_Add(Vars, CV, 'Sender')^.CV_Class := p;
    TStringListHack(Sender).IFS.RunScriptProc(E^.Cv_Proc,
      Vars);
  finally
    VM_Destroy(Vars);
  end;
end;

procedure TEventsClass.OnChanging(Sender: TObject);
var
  p: PCreatedClass;
  Vars: PVariableManager;
  E, CV: PIfVariant;
begin
  p := Pointer(TStringListHack(Sender).Tag);
  if not assigned(p) then exit;
  if not GetClassVariable2(p, p^.ClassType^.Ext, 'FONCHANGING', E, True) then
    exit;
  if not assigned(E^.Cv_Proc) then exit;
  Vars := VM_Create(nil);
  try
    CV := CreateCajVariant(p^.ClassType);
    Vm_Add(Vars, CV, 'Sender')^.CV_Class := p;
    TStringListHack(Sender).IFS.RunScriptProc(E^.Cv_Proc,
      Vars);
  finally
    VM_Destroy(Vars);
  end;
end;

function StrListProc(Sender: TIfPasScript; ScriptID: Pointer; proc: PProcedure;
  Params: PVariableManager; res: PIfVariant): TIfPasScriptError;
var
  p, Self, Strs, E, Obj: PIfVariant;
  U: TEventsClass;
begin
  Self := Vm_Get(Params, 0);
  StrListProc := ENoError;
  if not GetClassVariable2(Self^.CV_Class, proc^.ClassType^.Ext, 'DATA', p, True)
    then
  begin
    StrListProc := ENotSupported;
    exit;
  end;
  if proc^.Name = '!CREATE' then
  begin
    p^.Cv_Int1 := TStringListHack.Create;
    TStringListHack(p^.Cv_Int1).Tag := Self^.CV_Class;
    Sender.AddResource(TStrListFree, p^.Cv_Int1);
    TStringListHack(p^.Cv_Int1).IFS := Sender;
    U := NewEventsClass(Sender);
    TStringListHack(p^.Cv_Int1).OnChange := U.OnChange;
    TStringListHack(p^.Cv_Int1).OnChanging := U.OnChanging;
  end
  else
    if proc^.Name = '!DESTROY' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      Sender.RemoveResource(p^.Cv_Int1);
      TStringListHack(p^.Cv_Int1).Free;
      p^.Cv_Int1 := nil;
      p^.Cv_Int2 := nil;
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!CLEAR' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        TStringListHack(p^.Cv_Int1).Clear;
      except
        on E: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!NAMES' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        SetString(res,
          TStringListHack(p^.Cv_Int1).Names[(GetInteger(GetVarLink(Vm_Get(Params,
            1))))]);
      except
        on E: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!GETITEM' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        SetString(res,
          TStringListHack(p^.Cv_Int1)[(GetInteger(GetVarLink(Vm_Get(Params,
            1))))]);
      except
        on E: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!SETITEM' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        TStringListHack(p^.Cv_Int1)[(GetInteger(GetVarLink(Vm_Get(Params, 1))))]
          :=
          GetString(GetVarLink(Vm_Get(Params, 2)));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!GETVALUES' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        SetString(res,
          TStringListHack(p^.Cv_Int1).Values[(GetString(GetVarLink(Vm_Get(Params,
            1))))]);
      except
        on E: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!SETVALUES' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        TStringListHack(p^.Cv_Int1).Values[(GetString(GetVarLink(Vm_Get(Params,
          1))))]
          :=
          GetString(GetVarLink(Vm_Get(Params, 2)));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!GETSORTED' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      SetBoolean(res, TStringListHack(p^.Cv_Int1).Sorted);
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!SETSORTED' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      TStringListHack(p^.Cv_Int1).Sorted :=
        GetBoolean(GetVarLink(Vm_Get(Params, 1)));
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!EXCHANGE' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        TStringListHack(p^.Cv_Int1).Exchange(
          GetInteger(GetVarLink(Vm_Get(Params, 1))),
          GetInteger(GetVarLink(Vm_Get(Params, 2))));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!FIND' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      SetBoolean(Res, TStringListHack(p^.Cv_Int1).Find(
        GetString(GetVarLink(Vm_Get(Params, 1))),
        GetVarLink(Vm_Get(Params, 2)).CV_SInt32));
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!INDEXOF' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      SetInteger(Res, TStringListHack(p^.Cv_Int1).IndexOf(
        GetString(GetVarLink(Vm_Get(Params, 1)))));
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!INSERT' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        TStringListHack(p^.Cv_Int1).Insert(
          GetInteger(GetVarLink(Vm_Get(Params, 1))),
          GetString(GetVarLink(Vm_Get(Params, 2))));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!INDEXOF' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      SetInteger(Res, TStringListHack(p^.Cv_Int1).IndexOfName(
        GetString(GetVarLink(Vm_Get(Params, 1)))));
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!MOVE' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        TStringListHack(p^.Cv_Int1).Move(
          GetInteger(GetVarLink(Vm_Get(Params, 1))),
          GetInteger(GetVarLink(Vm_Get(Params, 2))));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!SORT' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      TStringListHack(p^.Cv_Int1).Sort;
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!GETTEXT' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      SetString(res,
        TStringListHack(p^.Cv_Int1).Text);
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!ADDSTRINGS' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      if (not GetClassVariable(Vm_Get(Params, 1),
        Sender.GetType('TSTRINGLIST')^.Ext,
        'DATA', Strs, True)) or (Strs^.Cv_Int1 = nil) then
      begin
        StrListProc := ENotSupported;
        Exit;
      end;
      if not (TOBJECT(Strs^.CV_Int1) is TStringListHack) then
      begin
        StrListProc := ENotSupported;
        Exit;
      end;
      try
        TStringListHack(p^.Cv_Int1).AddStrings(TStringListHack(Strs^.Cv_Int1));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!EQUALS' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      if (not GetClassVariable(Vm_Get(Params, 1),
        Sender.GetType('TSTRINGLIST')^.Ext,
        'DATA', Strs, True)) or (Strs^.Cv_Int1 = nil) then
      begin
        StrListProc := ENotSupported;
        Exit;
      end;
      if not (TOBJECT(Strs^.CV_Int1) is TStringListHack) then
      begin
        StrListProc := ENotSupported;
        Exit;
      end;
      try
        SetBoolean(Res,
          TStringListHack(p^.Cv_Int1).Equals(TStringListHack(Strs^.Cv_Int1)));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!ADDOBJECT' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        Obj := Vm_Get(Params, 2);
        if (Obj.VType.atypeid = CSV_Class) and Assigned(Obj^.CV_Class) then
          SetInteger(Res,
            TStringListHack(p^.Cv_Int1).AddObject(GetString(GetVarLink(Vm_Get(Params, 1))),
            Pointer(Vm_Get(Params, 2)^.CV_Class)))
        else
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, 'Not a valid TObject type.');
        end;
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!INSERTOBJECT' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        Obj := Vm_Get(Params, 3);
        if (Obj.VType.atypeid = CSV_Class) and Assigned(Obj^.CV_Class) then
          TStringListHack(p^.Cv_Int1).InsertObject(GetInteger(GetVarLink(Vm_Get(Params, 1))),
            GetString(GetVarLink(Vm_Get(Params, 2))),
            Pointer(Vm_Get(Params, 3)^.CV_Class))
        else
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, 'Not a valid TObject type.');
        end;
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!INDEXOFOBJECT' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        Obj := Vm_Get(Params, 1);
        if (Obj.VType.atypeid = CSV_Class) and Assigned(Obj^.CV_Class) then
          SetInteger(Res,
            TStringListHack(p^.Cv_Int1).IndexOfObject(Pointer(Vm_Get(Params,
            3)^.CV_Class)))
        else
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, 'Not a valid TObject type.');
        end;
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!GETOBJECT' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        Res^.CV_Class :=
          Pointer(TStringListHack(p^.Cv_Int1).Objects[GetInteger(GetVarLink(Vm_Get(Params,
            1)))]);
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!SETOBJECT' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        Obj := Vm_Get(Params, 2);
        if (Obj.VType.atypeid = CSV_Class) and Assigned(Obj^.CV_Class) then
          TStringListHack(p^.Cv_Int1).Objects[GetInteger(GetVarLink(Vm_Get(Params, 1)))] :=
            TObject(Vm_Get(Params, 2)^.CV_Class)
        else
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, 'Not a valid TObject type.');
        end;
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
{$IFNDEF NOSTREAMS}
    if proc^.Name = '!LOADFROMSTREAM' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      if (not GetClassVariable(Vm_Get(Params, 1),
        Sender.GetType('TSTREAM')^.Ext,
        'DATA', Strs, True)) or (Strs^.Cv_Int1 = nil) then
      begin
        StrListProc := ENotSupported;
        Exit;
      end;
      if not (TOBJECT(Strs^.CV_Int1) is TStream) then
      begin
        StrListProc := ENotSupported;
        Exit;
      end;
      try
        TStringListHack(p^.Cv_Int1).LoadFromStream(TStream(Strs^.CV_Int1));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!SAVETOSTREAM' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      if (not GetClassVariable(Vm_Get(Params, 1),
        Sender.GetType('TSTREAM')^.Ext,
        'DATA', Strs, True)) or (Strs^.Cv_Int1 = nil) then
      begin
        StrListProc := ENotSupported;
        Exit;
      end;
      if not (TOBJECT(Strs^.CV_Int1) is TStream) then
      begin
        StrListProc := ENotSupported;
        Exit;
      end;
      try
        TStringListHack(p^.Cv_Int1).SaveToStream(TStream(Strs^.CV_Int1));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
{$ENDIF}
    if proc^.Name = '!ASSIGN' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      if (not GetClassVariable(Vm_Get(Params, 1),
        Sender.GetType('TSTRINGLIST')^.Ext,
        'DATA', Strs, True)) or (Strs^.Cv_Int1 = nil) then
      begin
        StrListProc := ENotSupported;
        Exit;
      end;
      if not (TOBJECT(p^.Cv_Int1) is TStringListHack) then
      begin
        StrListProc := ENotSupported;
        Exit;
      end;
      try
        TStringListHack(p^.Cv_Int1).Assign(TStringListHack(Strs^.Cv_Int1));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!SETTEXT' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        TStringListHack(p^.Cv_Int1).Text :=
          GetString(GetVarLink(Vm_Get(Params, 1)));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!CUSTOMSORT' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      if not GetClassVariable2(Self^.CV_Class, proc^.ClassType^.Ext,
        'FCUSTOMSORT', E, True)
        then
      begin
        StrListProc := ENotSupported;
        exit;
      end;
      E.Cv_Proc := GetVarLink(Vm_Get(Params, 1)).Cv_Proc;
      try
        TStringList(p^.Cv_Int1).CustomSort(StrCustomSort);
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
{$IFNDEF NOFILEACCESS}
    if proc^.Name = '!LOADFROMFILE' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        TStringListHack(p^.Cv_Int1).LoadFromFile(GetString(GetVarLink(Vm_Get(Params,
          1))));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!SAVETOFILE' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        TStringListHack(p^.Cv_Int1).SaveToFile(GetString(GetVarLink(Vm_Get(Params,
          1))));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
{$ENDIF}
    if proc^.Name = '!GETCOMMATEXT' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      SetString(res, TStringListHack(p^.Cv_Int1).CommaText);
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!SETCOMMATEXT' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        TStringListHack(p^.Cv_Int1).CommaText :=
          GetString(GetVarLink(Vm_Get(Params, 1)));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!COUNT' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      SetInteger(res, TStringListHack(p^.Cv_Int1).Count);
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!ADD' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        SetInteger(Res,
          TStringListHack(p^.Cv_Int1).Add(GetString(GetVarLink(Vm_Get(Params,
          1)))));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!APPEND' then
  begin

    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        TStringListHack(p^.Cv_Int1).Append(GetString(GetVarLink(Vm_Get(Params,
          1))));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!DELETE' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        TStringListHack(p^.Cv_Int1).Delete(GetInteger(GetVarLink(Vm_Get(Params,
          1))));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!GETDUPLICATES' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        SetInteger(Res, Ord(TStringListHack(p^.Cv_Int1).Duplicates));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    if proc^.Name = '!SETDUPLICATES' then
  begin
    if Sender.IsValidResource(TStrListFree, p^.Cv_Int1) then
    begin
      try
        TStringListHack(p^.Cv_Int1).Duplicates :=
          TDuplicates(GetInteger(GetVarLink(VM_GET(Params, 1))));
      except
        on e: Exception do
        begin
          StrListProc := ECustomError;
          Sender.RunError2(Sender, ECustomError, E.Message);
        end;
      end
    end
    else
      StrListProc := EClassNotCreated; // EInternalError
  end
  else
    StrListProc := EUnknownIdentifier;
end;

procedure RegisterTStringList(p: TIfPasScript);
begin
  p.AddVariable('dupIgnore', 'Integer', True).CV_SInt32 := Ord(dupIgnore);
  p.AddVariable('dupAccept', 'Integer', True).CV_SInt32 := Ord(dupAccept);
  p.AddVariable('dupError', 'Integer', True).CV_SInt32 := Ord(dupError);
// Can't declare TStringList in a type before the TStringList itself
  p.AddType('TStringListSortCompare',
    'function(List: TObject; Index1, Index2: Integer): Integer;');
  p.AddType('TNotifyEvent', 'procedure(Sender: TObject);');
{$IFNDEF NOSTREAMS}
  RegisterStreams(p);
{$ENDIF}
  p.AddClass('TStringList', TStringListT, @StrListProc);
  p.AddType('TStrings', 'TStringList');
end;

initialization
  EventsClasses := TList.Create;
finalization
  FreeEventsClasses(nil);
  EventsClasses.Free;
end.

